@@ -13,17 +13,18 @@ Interface Functions:
     iter_upoints
     iter_terms
 
+    vpoint2point
+
 Interface Classes:
     Variable
     Function
-    Slicer
-    VectorFunction
 """
 
 import collections
 import functools
+import operator
 
-from pyeda.util import bit_on, boolify, cached_property
+from pyeda.util import bit_on, cached_property
 
 VARIABLES = dict()
 
@@ -81,13 +82,13 @@ def num2upoint(num, vs):
         upoint[bit_on(num, i)].add(v.uniqid)
     return frozenset(upoint[0]), frozenset(upoint[1])
 
-def num2term(num, vs, conj=False):
+def num2term(num, fs, conj=False):
     """Convert a number into a min/max term.
 
     Parameters
     ----------
     num : int
-    vs : [Variable]
+    fs : [Function]
     conj : bool
         conj=False for minterms, conj=True for maxterms
 
@@ -110,9 +111,9 @@ def num2term(num, vs, conj=False):
     +-------+----------+----------+
     """
     if conj:
-        return tuple(-v if bit_on(num, i) else v for i, v in enumerate(vs))
+        return tuple(~f if bit_on(num, i) else f for i, f in enumerate(fs))
     else:
-        return tuple(v if bit_on(num, i) else -v for i, v in enumerate(vs))
+        return tuple(f if bit_on(num, i) else ~f for i, f in enumerate(fs))
 
 def point2upoint(point):
     """Convert a point into an untyped point.
@@ -136,9 +137,9 @@ def point2term(point, conj=False):
     point : {Variable: int}
     """
     if conj:
-        return tuple(-v if val else v for v, val in point.items())
+        return tuple(~v if val else v for v, val in point.items())
     else:
-        return tuple(v if val else -v for v, val in point.items())
+        return tuple(v if val else ~v for v, val in point.items())
 
 def iter_points(vs):
     """Iterate through all points in an N-dimensional space.
@@ -160,15 +161,33 @@ def iter_upoints(vs):
     for num in range(1 << len(vs)):
         yield num2upoint(num, vs)
 
-def iter_terms(vs, conj=False):
+def iter_terms(fs, conj=False):
     """Iterate through all min/max terms in an N-dimensional space.
 
     Parameters
     ----------
-    vs: [Variable]
+    fs : [Function]
     """
-    for num in range(1 << len(vs)):
-        yield num2term(num, vs, conj)
+    for num in range(1 << len(fs)):
+        yield num2term(num, fs, conj)
+
+def vpoint2point(vpoint):
+    """Convert a vector point to a point."""
+    point = dict()
+    for v, val in vpoint.items():
+        point.update(_flatten(v, val))
+    return point
+
+def _flatten(v, val):
+    """Recursively flatten vectorized var => {0, 1} mappings."""
+    if isinstance(v, Variable):
+        yield v, int(val)
+    else:
+        if len(v) != len(val):
+            raise ValueError("expected 1:1 mapping from Variable => {0, 1}")
+        for _var, _val in zip(v, val):
+            for __var, __val in _flatten(_var, _val):
+                yield __var, __val
 
 
 _UNIQIDS = dict()
@@ -228,11 +247,11 @@ class Function(object):
     """
 
     # Operators
-    def __neg__(self):
+    def __invert__(self):
         """Boolean negation operator
 
         +-----------+------------+
-        | :math:`f` | :math:`-f` |
+        | :math:`f` | :math:`f'` |
         +===========+============+
         |         0 |          1 |
         +-----------+------------+
@@ -241,8 +260,8 @@ class Function(object):
         """
         raise NotImplementedError()
 
-    def __add__(self, other):
-        """Boolean disjunction (addition, OR) operator
+    def __or__(self, other):
+        """Boolean disjunction (sum, OR) operator
 
         +-----------+-----------+---------------+
         | :math:`f` | :math:`g` | :math:`f + g` |
@@ -258,18 +277,11 @@ class Function(object):
         """
         raise NotImplementedError()
 
-    def __radd__(self, other):
-        return self.__add__(other)
+    def __ror__(self, other):
+        return self.__or__(other)
 
-    def __sub__(self, other):
-        """Alias: a - b <=> a + -b"""
-        raise NotImplementedError()
-
-    def __rsub__(self, other):
-        return self.__neg__().__add__(other)
-
-    def __mul__(self, other):
-        r"""Boolean conjunction (multiplication, AND) operator
+    def __and__(self, other):
+        r"""Boolean conjunction (product, AND) operator
 
         +-----------+-----------+-------------------+
         | :math:`f` | :math:`g` | :math:`f \cdot g` |
@@ -285,10 +297,10 @@ class Function(object):
         """
         raise NotImplementedError()
 
-    def __rmul__(self, other):
-        return self.__mul__(other)
+    def __rand__(self, other):
+        return self.__and__(other)
 
-    def xor(self, other):
+    def __xor__(self, other):
         r"""Boolean exclusive or (XOR) operator
 
         +-----------+-----------+--------------------+
@@ -305,6 +317,14 @@ class Function(object):
         """
         raise NotImplementedError()
 
+    # Deprecated operators
+    def __sub__(self, other):
+        """Alias: a - b <=> a + -b"""
+        raise NotImplementedError()
+
+    def __rsub__(self, other):
+        return self.__invert__().__or__(other)
+
     @property
     def support(self):
         r"""Return the support set of a function.
@@ -382,7 +402,7 @@ class Function(object):
 
     def vrestrict(self, vpoint):
         """Expand all vectors before applying 'restrict'."""
-        return self.restrict(_expand_vectors(vpoint))
+        return self.restrict(vpoint2point(vpoint))
 
     def compose(self, mapping):
         r"""
@@ -413,11 +433,11 @@ class Function(object):
         """Iterate through the cofactors of N variables.
 
         The *cofactor* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)`
-        with respect to variable :math:`x_i` is
+        with respect to variable :math:`x_i` is:
         :math:`f_{x_i} = f(x_1, x_2, ..., 1, ..., x_n)`
 
         The *cofactor* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)`
-        with respect to variable :math:`x_i'` is
+        with respect to variable :math:`x_i'` is:
         :math:`f_{x_i'} = f(x_1, x_2, ..., 0, ..., x_n)`
         """
         vs = self._expect_vars(vs)
@@ -428,64 +448,41 @@ class Function(object):
         """Return a tuple of cofactors of N variables.
 
         The *cofactor* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)`
-        with respect to variable :math:`x_i` is
+        with respect to variable :math:`x_i` is:
         :math:`f_{x_i} = f(x_1, x_2, ..., 1, ..., x_n)`
 
         The *cofactor* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)`
-        with respect to variable :math:`x_i'` is
+        with respect to variable :math:`x_i'` is:
         :math:`f_{x_i'} = f(x_1, x_2, ..., 0, ..., x_n)`
         """
         return tuple(cf for cf in self.iter_cofactors(vs))
 
-    def is_neg_unate(self, vs=None):
-        r"""Return whether a function is negative unate.
-
-        A function :math:`f(x_1, x_2, ..., x_i, ..., x_n)` is *negative unate*
-        in variable :math:`x_i` if :math:`f_{x_i'} \geq f_{xi}`.
-        """
-        raise NotImplementedError()
-
-    def is_pos_unate(self, vs=None):
-        r"""Return whether a function is positive unate.
-
-        A function :math:`f(x_1, x_2, ..., x_i, ..., x_n)` is *positive unate*
-        in variable :math:`x_i` if :math:`f_{x_i} \geq f_{x_i'}`.
-        """
-        raise NotImplementedError()
-
-    def is_binate(self, vs=None):
-        """Return whether a function is binate.
-
-        A function :math:`f(x_1, x_2, ..., x_i, ..., x_n)` is *binate* in
-        variable :math:`x_i` if it is neither negative nor positive unate in
-        :math:`x_i`.
-        """
-        return not (self.is_neg_unate(vs) or self.is_pos_unate(vs))
-
     def smoothing(self, vs=None):
         """Return the smoothing of a function.
 
         The *smoothing* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)` with respect
-        to variable :math:`x_i` is :math:`S_{x_i}(f) = f_{x_i} + f_{x_i'}`.
+        to variable :math:`x_i` is:
+        :math:`S_{x_i}(f) = f_{x_i} + f_{x_i'}`
         """
-        return functools.reduce(self.__class__.__add__, self.iter_cofactors(vs))
+        return functools.reduce(operator.or_, self.iter_cofactors(vs))
 
     def consensus(self, vs=None):
         r"""Return the consensus of a function.
 
         The *consensus* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)` with respect
-        to variable :math:`x_i` is :math:`C_{x_i}(f) = f_{x_i} \cdot f_{x_i'}`.
+        to variable :math:`x_i` is:
+        :math:`C_{x_i}(f) = f_{x_i} \cdot f_{x_i'}`
         """
-        return functools.reduce(self.__class__.__mul__, self.iter_cofactors(vs))
+        return functools.reduce(operator.and_, self.iter_cofactors(vs))
 
     def derivative(self, vs=None):
         r"""Return the derivative of a function.
 
         The *derivative* of :math:`f(x_1, x_2, ..., x_i, ..., x_n)` with respect
-        to variable :math:`x_i` is
-        :math:`\frac{\partial}{\partial x_i} f = f_{x_i} \oplus f_{x_i'}`.
+        to variable :math:`x_i` is:
+        :math:`\frac{\partial}{\partial x_i} f = f_{x_i} \oplus f_{x_i'}`
         """
-        return functools.reduce(self.__class__.xor, self.iter_cofactors(vs))
+        return functools.reduce(operator.xor, self.iter_cofactors(vs))
 
     def is_zero(self):
         """Return whether this function is zero.
@@ -504,7 +501,7 @@ class Function(object):
         raise NotImplementedError()
 
     @staticmethod
-    def box(arg):
+    def box(obj):
         """Convert primitive types to a Function."""
         raise NotImplementedError()
 
@@ -532,144 +529,3 @@ class Function(object):
                 raise TypeError("expected iter of Variable")
 
 
-class Slicer(object):
-    """Interface for vector objects that supports non-zero start index.
-
-    Similar to a Python list, this class can be used to support arbitrarily
-    nested vectors. Unlike Python lists, you can use a Slicer object to
-    instantiate a vector with arbitrary start, stop indices.
-
-    NOTE: This is a general-purpose utility class, but for the purposes of
-          creating and manipulating BitVectors, we recommend using the
-          BitVector class from pyeda.vexpr, or the 'bitvec' convenience
-          function.
-    """
-
-    def __init__(self, items, start=0):
-        self.items = items
-        self.start = start
-
-    @property
-    def stop(self):
-        """Return the stop index."""
-        return self.start + self.__len__()
-
-    def __iter__(self):
-        return iter(self.items)
-
-    def __len__(self):
-        return len(self.items)
-
-    def getifz(self, i):
-        """Get item from zero-based index."""
-        return self.__getitem__(i + self.start)
-
-    def __getitem__(self, sl):
-        if isinstance(sl, int):
-            return self.items[self._norm_idx(sl)]
-        else:
-            items = self.items[self._norm_slice(sl)]
-            return self.__class__(items, sl.start)
-
-    def __setitem__(self, sl, item):
-        if isinstance(sl, int):
-            self.items[self._norm_idx(sl)] = item
-        else:
-            self.items[self._norm_slice(sl)] = item
-
-    def __delitem__(self, sl):
-        if isinstance(sl, int):
-            del self.items[self._norm_idx(sl)]
-        else:
-            del self.items[self._norm_slice(sl)]
-
-    def _norm_idx(self, i):
-        """Return an index normalized to vector start index."""
-        if i >= self.start and i < self.stop:
-            idx = i - self.start
-        elif i >= -self.stop and i < -self.start:
-            idx = i + self.stop
-        else:
-            raise IndexError("list index out of range")
-        return idx
-
-    def _norm_slice(self, sl):
-        """Return a slice normalized to vector start index."""
-        limits = {'start': None, 'stop': None}
-        for k in ('start', 'stop'):
-            i = getattr(sl, k)
-            if i is not None:
-                if i >= self.start and i < self.stop:
-                    limits[k] = i - self.start
-                elif i >= -self.stop and i < -self.start:
-                    limits[k] = i + self.stop
-                else:
-                    raise IndexError("list index out of range")
-        return slice(limits['start'], limits['stop'])
-
-
-class VectorFunction(Slicer):
-    """
-    Abstract base class that defines an interface for a vector Boolean function.
-    """
-    def restrict(self, point):
-        """
-        Return the vector that results from applying the 'restrict' method to
-        all functions.
-        """
-        items = [f.restrict(point) for f in self]
-        return self.__class__(items, self.start)
-
-    def vrestrict(self, vpoint):
-        """Expand all vectors before applying 'restrict'."""
-        return self.restrict(_expand_vectors(vpoint))
-
-    def to_uint(self):
-        """Convert vector to an unsigned integer, if possible."""
-        num = 0
-        for i, f in enumerate(self):
-            if f.is_zero():
-                pass
-            elif f.is_one():
-                num += 1 << i
-            else:
-                fstr = "expected all functions to be a constant (0 or 1) form"
-                raise ValueError(fstr)
-        return num
-
-    def to_int(self):
-        """Convert vector to an integer, if possible."""
-        num = self.to_uint()
-        if self.items[-1].unbox():
-            return num - (1 << self.__len__())
-        else:
-            return num
-
-    def zext(self, num):
-        """Zero extend this vector by N bits."""
-        self.items += [0] * num
-
-    def sext(self, num):
-        """Sign extend this vector by N bits."""
-        bit = self.items[-1]
-        self.items += [bit] * num
-
-    def append(self, f):
-        """Append a function to the end of this vector."""
-        self.items.append(f)
-
-
-def _expand_vectors(vpoint):
-    """Expand all vectors in a substitution dict."""
-    point = dict()
-    for vf, vals in vpoint.items():
-        if isinstance(vf, VectorFunction):
-            if len(vf) != len(vals):
-                fstr = ("invalid vector point: "
-                        "expected 1:1 mapping from VectorFunction => {0, 1}")
-                raise ValueError(fstr)
-            for i, val in enumerate(vals):
-                point[vf.getifz(i)] = boolify(val)
-        else:
-            point[vf] = boolify(vals)
-    return point
